#dyn.load("TFlocs.so")
## turn a peak list into a 0/1 vector
peakLocs <- function(thispeak, winsize=1000, chrlen) {
  #library.dynam("TFlocs.so",package=c("ChIPCor"))
  n0.chr=sapply(chrlen, function(x) ceiling(x/winsize))
  thisres=rep.int(0, sum(n0.chr))
  allchrs=names(chrlen)
  idx=split(1:nrow(thispeak), thispeak$chr)
  idx=idx[allchrs]
  counter=0
  for(ichr in seq(along=allchrs)) {
    thischr=allchrs[ichr]
    if(is.null(idx[[thischr]])) { ## no peak on this chr
      counter = counter +n0.chr[ichr]
      next
    }
    thischrlen=chrlen[thischr]
    thispeak.chr=thispeak[idx[[thischr]],]
    res=rep.int(0, n0.chr[ichr])
    z <- .C("TFlocs", as.integer(floor(thispeak.chr[,"start"]/winsize)),
            as.integer(floor(thispeak.chr[,"end"]/winsize)), result=as.integer(res),
            as.integer(nrow(thispeak.chr)))
    tmpidx=counter+(1:n0.chr[ichr])
    thisres[tmpidx] = z$result
    counter = counter + n0.chr[ichr]
  }
  thisres
}
#segment the genome into winsize bins according to input bed file of segmentation
#a peak list can be treated as a segmentation file with only two states 0 and 1 (peak region)
genomeSeg<-function(segbedfilename,winsize=1000, chrlen)
{
	####################################
	#### get segmentation bed file
	#### read in segmentation bed file
	tmp=read.table(segbedfilename, sep="\t")[,1:4]
	colnames(tmp)=c("chr","start", "end", "state")
	ix=split(1:nrow(tmp), tmp[,4])
	## total length of each segmentation
	ll=tmp$end-tmp$start
	seg.len=tapply(ll, tmp[,4], sum)

	## generate segmentation, care when a bin belong to several states.
	## need to prioritize the states
	nam=names(seg.len)
	nam2=gsub("_", " ", nam)
	idx.state=as.integer(substr(nam2, 1,2))
	nam=rep("",length(seg.len))
	nam[idx.state]=nam2
	nam=c("None", nam)
	seg.len[idx.state]=seg.len
	names(seg.len)=nam[-1]

	#number of bins for each chromosome
	n0.chr=sapply(chrlen, function(x) ceiling(x/winsize))
	#record the segmentation state for each winsize(bp) bin
	peakloc.mat=matrix(0, nrow=sum(n0.chr), ncol=length(ix))
	for(i in 1:length(ix)) {
	  thispeak=tmp[ix[[i]],]
	  peakloc.mat[,idx.state[i]]=peakLocs(thispeak, winsize, chrlen)
	}
	colnames(peakloc.mat)=nam[-1]

	seg.result=rep(0, sum(n0.chr))
	for(i in rev(1:ncol(peakloc.mat))) {
	  seg.result[peakloc.mat[,i]==1]=i
	}
	seg.result
}
mytable <- function(x1, x2) {
  res=matrix(0,nrow=2, ncol=2)
  ix1=x1==1
  ix2=x2==1
  res[1,1]=sum(!ix1 & !ix2)
  res[1,2]=sum(!ix1 & ix2)
  res[2,1]=sum(ix1 & !ix2)
  res[2,2]=sum(ix1 & ix2)
  res
}
##### compute overlap statistics for a pair of factors
pairFactor <- function(tf1, tf2, seg, tf1.nam, tf2.nam) {
  nseg=max(seg)
  tbls=vector("list", nseg)
  for(i in 1:nseg) {
    iseg=seg==i
    tbls[[i]]=mytable(tf1[iseg], tf2[iseg])
  }
  list(tb=tbls, TFname1=tf1.nam, TFname2=tf2.nam)
}
prepareTbls<-function(peakloc.mat,seg.result)
{
	n0=ncol(peakloc.mat)
	tbls=vector("list", n0*(n0-1)/2)
	counter=1
	for(i in 1:(n0-1)) {
	  cat("processing TF pair", i, ": ")
	  for(j in (i+1):n0) {
		cat(j, ",")
		tbls[[counter]]=pairFactor(peakloc.mat[,i], peakloc.mat[,j], seg.result, 
					colnames(peakloc.mat)[i], colnames(peakloc.mat)[j])
		counter=counter+1
	  }
	  cat("\n")
	}
	tbls
}
ChIPCor<-function(tbls)
{	
	nseg=length(tbls[[1]]$tb)
	## compute expected proportion of overlap from a 2x2 table
	compute.expP <- function(tt) {
	  N <- sum(tt)
	  p1=sum(tt[2,])/N
	  q1=sum(tt[,2])/N
	  c(p1*q1*N,tt[2,2])
	}
	
	exp=obs=matrix(0, nrow=nseg, ncol=length(tbls))
	for(i in 1:length(tbls)) {
		overlaps<-sapply(1:nseg, function(l) compute.expP(tbls[[i]]$tb[[l]]))
		overlaps<-t(overlaps)
		exp[,i]=overlaps[,1]
		obs[,i]=overlaps[,2]
	}
	
	################################
	## conduct correction
	################################
	## estimate baseline
	c0=rep(0, nseg)
	for(i in 1:nseg) {
	  c0[i]=mean(obs[i,exp[i,]<1])
	}
	
	## function to calculate true expected 
	correct <- function(x, fit) {
	  ix=sort(x, index.return=TRUE)$ix
	  res=x
	  res[ix]=fit$y
	  exp(res)
	}
	## now compute Expected
	ll=NULL
	for(i in 1:nseg) {
		E=exp[i,];  O=obs[i,]
		ix=(E>1) & (O>0)
		X=log(E[ix]); Y=log(O[ix])
		ll[[i]]=lowess(X,Y)
		newE=E
		newE[!ix]=c0[i]
		newE[ix]=correct(X, ll[[i]])
	}
	##### compute test statistics
	## compute var of overlaps
	compute.Var <- function(N, p) {
	  N*p*(1-p)
	}
	## compute segment-wise t
	tstat=matrix(0, ncol=nseg, nrow=ncol(obs))
	for(i in 1:nseg) {
	  N=sum(tbls[[1]]$tb[[i]])
	  E=exp[i,];  O=obs[i,]
	  ix=(E>1) & (O>0)
	  newE=E
	  newE[!ix]=c0[i]
	  newE[ix]=correct(E[ix], ll[[i]])
	  p=newE/N
	  Vx=compute.Var(N, p)
	  tstat[,i]=(O-newE)/sqrt(Vx)
	}
	
	## construct test statistics
	chistat=rowSums(tstat[,1:nseg]^2)
	
	## construct null distribution based on left tail
	set.seed(2)
	t0=matrix(0, nrow=10000, ncol=nseg)
	for(i in 1:nseg) {
	  thist=tstat[,i]
	  t0[,i]=sample(thist[thist<0], 10000, replace=TRUE)
	}
	chistat0=rowSums(t0^2)
	
	
	###check statistics
	p.propose<-sapply(1:length(chistat), function(i) length(which(chistat0>chistat[i]))/length(chistat0))
	
	library(qvalue)
	###control for multiplicity
	qobj.proposed<-qvalue(p=p.propose)
	qvalues.proposed <- qobj.proposed$qvalues
	pi0.proposed <- qobj.proposed$pi0
	
	TF1name<-sapply(1:length(tbls), function(i) tbls[[i]]$TFname1)
	TF2name<-sapply(1:length(tbls), function(i) tbls[[i]]$TFname2)
	result<-data.frame(TF1=TF1name, TF2=TF2name,qvalue=qvalues.proposed)
}