\name{ChIPCor}
\alias{ChIPCor}
\title{Measuring the Spatial Correlations of Protein Binding Sites}
\description{This function evaluates the significance of overlapping from a pair of proteins by leveraging information from publicly available data. It accounts for background artifacts and genome heterogeneity.}

\usage{
ChIPCor(tbls)
}

\arguments{
\item{tbls}{a list containing overlapping information of all available pairs of proteins for the given biological condition, either from public data repository or the ones under investigation. Each element contains the names of a given pair of proteins and a list of two by two tables representing their overlapping at each segment type. Details of constructing the list from raw bed files can be found in function prepareTbls.}
}

\details{By borrowing information from protein binding profiles from public data repositories, the proposed method corrects for background artifacts involved in ChIP-seq data and the Simpson's paradox phenomenon due to genome heterogeneity.
}

\value{
\item{TF1}{the name of the first protein of a given pair of proteins}
\item{TF2}{the name of the first protein of a given pair of proteins}
\item{qvalue}{the resulting qvalues for each pair of tested proteins}
}

\author{Yingying Wei, Hao Wu}

\references{Wei, Y., Wu, H.(2015) Measuring the Spatial Correlations of Protein Binding Sites.}

\examples{
#protein pair overlapping information by chromHMM segmentsation 
#for K562 cell lines from the ENCODE project
data(K562tbls)

#data structure
str(tbls[[1]])

#Conduct testing
qval.K562<-ChIPCor(tbls)
head(qval.K562)

#protein pair overlapping information by chromHMM segmentsation 
#for GM12878 cell lines from the ENCODE project
data(GM12878tbls)

#Conduct testing
qval.GM12878<-ChIPCor(tbls)
head(qval.GM12878)


}

\keyword{Measuring Protein Binding Overlaps}